=head1 NAME

ADS::RefResolver::Client - ADS bibcode resolver PERL client

=head1 SYNOPSIS

  use ADS::RefResolver::Client;
  my $resolver = ADS::RefResolver::Client->new();
  my @refs = ('Abt, H. 1990, ApJ, 357, 1',
              'J. B. Gupta, and J. H. Hamilton, Phys. Rev. C 16, 427 (1977)',
              'Hermsen, W., et. al. 1992, IAU Circ. No. 5541',
              'Pollock, J. T. 1982, Ph. D. Thesis, University of Florida');
  @results = $resolver->resolve(@refs);
  foreach my $r (@results) {
      print $r->{bibcode} if ($r->{bibcode} and $r->{confidence} >= 1);
      print "\t", $r->{reference}, "\n";
  }

=head1 DESCRIPTION

ADS::RefResolver::Client is a class implementing the ADS bibcode resolver
as a PERL client to the Web-based ADS resolving server.

=head1 METHODS

The following methods are supported:

=over 4

=item $resolver = ADS::RefResolver::Client->new(OPTIONS)

Constructor for this class.  
OPTIONS is a hash of KEY => VALUE pairs.  
The supported options are:

=over 4

=item * Debug (defaults to 0)

set to 1 to print out debugging output

=item * HttpMethod (defaults to "GET")

method to be used when submitting the HTTP request to the server

=back

=item $resolver->resolve(ARRAY)

Takes an input array of strings and returns an array of hashes
containing the resolved results.  Each element of the returned array
is an anonymous hash containing the following fields:

  reference:  the original reference string
  bibcode:    the bibliographic code corresponding to the reference
  confidence: an integer number representing the degree of certainty

=back

=cut


package ADS::RefResolver::Client;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;
use LWP::UserAgent;
use Sys::Hostname;

@EXPORT = qw();
$VERSION = '0.02';

my $version = sprintf("%s/%d.%02d", __PACKAGE__,
                   q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);

# these are the default values for the resolver
my %Fields = (Confidence   => 1,     # not used so far
              Debug        => 0,
              HttpMethod  => "POST",
              _ResolverURL => "http://adsres.cfa.harvard.edu/cgi-bin/refcgi.py",
              _LWPAgent    => "");

sub new {
    my $self = shift;
    my $class = ref($self) || $self;
    $self = { %Fields, @_ };

    # now customize user agent to carry this module's signature
    my $ua = new LWP::UserAgent or 
	die __PACKAGE__, "-.new(): error initializing LWP user agent";
    my $user = $ENV{USER} || $ENV{LOGNAME} || "anonymous";
    my $host = hostname() || "localhost";
    $ua->agent($version ."; ". $ua->agent ."; ". $user .'@'. $host);
    $self->{_LWPAgent} = $ua;

    bless $self, $class;
}

sub resolve {
    my $self = shift;
    my $ninput = scalar(@_);
    my $max_urllen = 4096;
    my $url = $self->{_ResolverURL};
    my $method = $self->{HttpMethod};
    my $agent = $self->{_LWPAgent};

    warn $self, ": no input references specified!" unless ($ninput);
    warn $self, ": resolving $ninput input references\n" if ($self->{Debug});

    # break down queries in chunks to avoid http choking
    # we do this by composing query strings that are no longer
    # than a pre-set maximum string size ($max_urllen):
    my @queries = ();
    my $base = &_base_query_string();
    my $query = "";
    while (@_) {    
	my $record = &_encode_record(shift);
	if ($query and length($query) + length($record) > $max_urllen) {
	    push(@queries,$base . $query);
	    $query = "";
	}
	$query .= $record;
    }
    if ($query) {
	push(@queries,$base . $query);
    }
    my @output = ();

    while ($query = shift(@queries)) {
	warn $self, ": target url is ", $url, "\n" if ($self->{Debug});
	warn $self, ": method is ", $method, "\n" if ($self->{Debug});
	warn $self, ": content is ", $query, "\n" if ($self->{Debug} and 
							  $method eq 'POST');
	if ($method eq 'GET') { 
	    $url .= '?' . $query;
	    $query = "";
	}
	
	my $req = new HTTP::Request($method,$url);
	if ($query) {
	    $req->header('Content-Type','application/x-www-form-urlencoded');
	    $req->header('Content-Length',length($query));
	    $req->content($query);
	}
	my $res = $agent->request($req);

	die $self, ": HTTP request failed!\n" unless ($res->is_success);
	warn $self, ": content is: ", $res->content if ($self->{Debug});

	my @results = $self->parse_content(&_decode($res->content));
	push(@output,@results);
    }
    warn $self, ": mismatch between number of input references (",
    scalar(@_), ") and output records (", scalar(@output), ")"
	unless ($ninput == scalar(@output));

    return @output;
}

# parses the output from the ADS resolver server and returns
# an array of hashes (one for each input line)
sub parse_content {
    my $self = shift;
    my $contents = shift;
    my @results = ();

    $contents =~ s/^.*?<LI>//is;
    my @lines = split(/<LI>/i,$contents);

    # template line is:  "%s (confidence %d) from %s</LI>\n";
    while (@lines) {
        my $l = shift(@lines);
        if ($l =~ /^(\S+)\s+\(confidence\s+([\d-]+)\)\s+from\s+(.*?)<\/LI>/i) {
            push(@results, {bibcode => $1, confidence => $2, reference => $3});
            warn $self, ": parsed record `$l'" if ($self->{Debug});
        } else {
            warn $self, ": cannot parse record `$l'";
            push(@results, {reference => $l});
        }
    }
    return @results;
}


# a refquery consists of all input line specified as
# values of the parameter "resolvethose"
sub _base_query_string {
    return "resolvethose=";
}

sub _encode_record {
    my $r = shift;
    $r =~ s/\n+/ /g;
    $r =~ s/^\s+|\s+$//g;
    return &_urlencode("$r\n");
}

sub _urlencode {
    my $str = shift;
    my $pat = shift || '[\x00-\x2A/;<=>?\x7F-\xFF]';
         
    $str =~ s/($pat)/sprintf("%%%02lx",unpack('C',$1))/ge;
    return $str;
}

sub _decode {
    my $str = shift;
    my $ent = shift || 
        { gt => ">", lt => "<", quot => '"', apos => "'", amp => "&" };
    my $entrx = join("|",keys(%$ent));

    $str =~ s/&($entrx);/$ent->{$1}/g;
    return $str;
}

=head1 EXAMPLES

See the subdirectory examples and the test subdirectory t for 
examples on how to use the resolver.

=head1 PREREQUISITES

ADS::RefResolver::Client requires a recent version of perl (5.005 or higher)
and the libwww-perl package available from CPAN and http://www.linpro.no/lwp/

=head1 BUILDING/INSTALLING

Once the archive is unpacked, use these commands:

      perl Makefile.PL
      make
      make test
      make install

=head1 AUTHOR

Alberto Accomazzi, <aaccomazzi@cfa.harvard.edu>

=head1 SEE ALSO

perl(1), LWP(3)

=cut

1;

__END__
